

/**
 * @author Alex Van Buskirk
 *
 */
public class ConnectK {
	private ConnectKGUI view;
	private BoardModel currentBoard;
	private CKPlayer[] players = new CKPlayer[3];
	
	public ConnectK(BoardModel model, CKPlayer player1, CKPlayer player2){
		currentBoard = model;
		players[1] = player1;
		players[2] = player2;
	}
	
	public ConnectK(BoardModel model, CKPlayer player1, CKPlayer player2, ConnectKGUI view){
		this(model, player1, player2);
		this.view = view;
	}
	
	public byte play(){
		byte currentPlayer = 1;
		while(currentBoard.winner() == -1){
			long begin = System.currentTimeMillis();
			System.out.println("Starting GC at time: " + (System.currentTimeMillis() - begin));
			if(view != null)
				view.setStatus("Waiting on GC");
			Runtime.getRuntime().gc();
			System.out.println("GC finished at: " + (System.currentTimeMillis() - begin));
			if(view != null)
				view.setStatus("Waiting for player " + currentPlayer);
			java.awt.Point p;
			begin = System.currentTimeMillis();
			PlayerThread pf = new PlayerThread(players[currentPlayer], (BoardModel) currentBoard.clone(), 5000);
			pf.start();
			try {
				pf.join(Integer.MAX_VALUE);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
			p = pf.move;
			if(p == null || currentBoard.getSpace(p) != 0){
				System.err.println("Player " + currentPlayer
						+ " returned a bad move " + p
						+ " or went over time (" + (System.currentTimeMillis() - begin) + ").");
				return currentPlayer==(byte)1?(byte)2:(byte)1;
			}
			currentBoard = currentBoard.placePiece(p, currentPlayer);
			if(view != null)
				view.placePiece(currentBoard.lastMove, currentPlayer);
			currentPlayer = (byte) (currentPlayer == 1? 2 : 1);
		}
		if(view != null){
			if(currentBoard.winner() != 0){
				view.setStatus("Player " 
					+ currentBoard.winner()
					+ " (" + players[currentBoard.winner()].teamName + ")" + " wins");
				view.highlightSpaces(currentBoard.winningSpaces());
			}
			if(!currentBoard.hasMovesLeft()){
				view.setStatus("Draw");
			}
		}
		return currentBoard.winner();
	}

	public int width() {
		return currentBoard.width;
	}

	public int height() {
		return currentBoard.height;
	}
}

class PlayerThread extends Thread{
	java.awt.Point move = null;
	CKPlayer player;
	BoardModel model;
	int deadline;
	
	public PlayerThread(CKPlayer player, BoardModel model, int deadline) {
		this.player = player;
		this.model = model;
		this.deadline = deadline;
	}
	
	public void run(){
		try {
			move = player.getMove(model, 5000);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
